//(c)2010 by Texas Instruments Incorporated, All Rights Reserved.
/*  
 * ======== main.c ========
 * OLIMEXINO-5510 production test
 *
 * Designed by: eng. Penka Sotirova
 *
 * ----------------------------------------------------------------------------+
 * Please refer to the MSP430 USB API Stack Programmer's Guide,located
 * in the root directory of this installation for more details.
 * ---------------------------------------------------------------------------*/
 
#include <intrinsics.h>
#include <string.h>
#include "ext_test.h"
#include "adc.h"

#include "USB_config/descriptors.h"

#include "USB_API/USB_Common/device.h"
#include "USB_API/USB_Common/types.h"               //Basic Type declarations
#include "USB_API/USB_Common/usb.h"                 //USB - specific functions

#include "F5xx_F6xx_Core_Lib/HAL_UCS.h"             
#include "F5xx_F6xx_Core_Lib/HAL_PMM.h"             

#include "USB_API/USB_CDC_API/UsbCdc.h"
#include "usbConstructs.h"

VOID Init_Ports (VOID);
VOID Init_Clock (VOID);
volatile BYTE bCDCDataReceived_event = FALSE;   //Flag set by event handler to indicate data has been received into USB buffer
void Init_Timer1 (void);            

//=====================Definitions=========================
#define     BUFFER_SIZE     130
#define     STR_SIZE        100

#define     TIMER1_PERIOD   4000              //timer1 period in 1/32768 Hz
#define     LED1_ON          PJOUT |= BIT3
#define     LED1_OFF         PJOUT &= ~BIT3

#define     LED1_TOGGLE      PJOUT ^= BIT3

#define     BUT             (P2IN & BIT0)


#define     PRESSED         0
#define     RELEASED        1



char dataBuffer[BUFFER_SIZE];
char nl[2] = "\n";
char str[STR_SIZE];
unsigned char start_up_flag = 0;

/*----------------------------------------------------------------------------+
 | Main Routine                                                                |
 +----------------------------------------------------------------------------*/
VOID main (VOID)
{

  
    WDTCTL = WDTPW + WDTHOLD;     //Stop watchdog timer
    
//    Low_Power_Entering();
    
    P2REN |= BIT0;                //Enable BUT pullup
    P2OUT = 0x01;
    P2DIR = 0x00;
 
    Init_Ports();          
    Init_Timer1();        

    SetVCore(3);
    Init_Clock();				//Init clocks

    USB_init();                 //Init USB

    //Enable various USB event handling routines

    USB_setEnabledEvents(
        kUSB_VbusOnEvent + kUSB_VbusOffEvent + kUSB_receiveCompletedEvent
        + kUSB_dataReceivedEvent + kUSB_UsbSuspendEvent + kUSB_UsbResumeEvent +
        kUSB_UsbResetEvent);

    //See if we're already attached physically to USB, and if so, connect to it
    //Normally applications don't invoke the event handlers, but this is an exception.
	
    if (USB_connectionInfo() & kUSB_vbusPresent){
        if (USB_enable() == kUSB_succeed){
            USB_reset();
            USB_connect();
        }
    }

    __enable_interrupt();                           //Enable interrupts globally
    while (1)
    {
        BYTE ReceiveError = 0, SendError = 0;
        WORD count;
        //Check the USB state and loop accordingly
        switch (USB_connectionState())
        {
            case ST_USB_DISCONNECTED:
//                __bis_SR_register(LPM3_bits + GIE);                             //Enter LPM3 until USB is connected
//                __no_operation();
                break;

            case ST_USB_CONNECTED_NO_ENUM:
                break;

            case ST_ENUM_ACTIVE:
//                __bis_SR_register(LPM0_bits + GIE);                             //Enter LPM0 until awakened by an event handler

              if ((BUT) == PRESSED)  LED1_ON;                                     //stop LED blinking      
              
              if (bCDCDataReceived_event)                                         //wait to receive characters              
              {                                                                                                                                                                                                                                                     
                  char dataBuffer[BUFFER_SIZE] = "";
                  bCDCDataReceived_event = FALSE;                                 //Clear flag early -- just in case execution breaks
                                                                                  //below because of an error
                  count = cdcReceiveDataInBuffer((BYTE*)dataBuffer,               //Count has the number of bytes received into dataBuffer
                      BUFFER_SIZE,
                      CDC0_INTFNUM);
                  if (start_up_flag == 0)                
                  {                    
                    if (cdcSendDataInBackground("\r\n\r\nOLIMEXINO-5510 \r\n\r\nPressing button BUT will stop the blinking of LED1. There is always local echo! \r\n",strlen("\r\n\r\nOLIMEXINO-5510 \r\n\r\nPressing button BUT will stop the blinking of LED1. There is always local echo! \r\n"),CDC0_INTFNUM,1))  	//Echo is back to the host
                    {  
                      SendError = 0x01; //Something went wrong -- exit
                      break;                       
                    }
                    start_up_flag = 1;
                  } 
                  
                  else                                                                            
                  {                                           
                    strcat(dataBuffer,nl);
                    if (cdcSendDataInBackground((BYTE*)dataBuffer,count,CDC0_INTFNUM,1)){  	//Echo is sent back to the host
                      SendError = 0x01;                                       		        //Something went wrong -- exit
                      break;
                    }
                  }  
              }          
       
                break;                                        //break of USB case

            case ST_ENUM_SUSPENDED:
//                __bis_SR_register(LPM3_bits + GIE);         //Enter LPM3 until a resume or VBUS-off event
                break;

            case ST_ENUM_IN_PROGRESS:
                break;

            case ST_NOENUM_SUSPENDED:
//                __bis_SR_register(LPM3_bits + GIE);
                break;

            case ST_ERROR:
                _NOP();
                break;

            default:;
        }
        if (ReceiveError || SendError){
            //TO DO: User can place code here to handle error
           //TO DO: User can place code here to handle error
          if (cdcSendDataInBackground("\r\n SENDING ERROR\r\n",strlen("\r\n SENDING ERROR\r\n"),CDC0_INTFNUM,1));
        }
    }  //while(1)
}                               
  
// ======== Init_Clock ========

VOID Init_Clock (VOID)
{
    if (USB_PLL_XT == 2){
		#if defined (__MSP430F552x) || defined (__MSP430F550x )
			P5SEL |= 0x0C;                                        //enable XT2 pins for F5529
		#elif defined (__MSP430F563x_F663x)
			P7SEL |= 0x0C;
		#endif
                        
      int timeout = 5000;
//      P1DIR |= 0x01;                   //set P1.0 as output
//      P1SEL |= 0x01;                   //set P1.0 as ACLK function

      P5DIR &= ~(BIT5 + BIT4);
      P5SEL |= BIT5 + BIT4;           //enable XT1 pins
      
      UCSCTL6 |= XCAP0 + XCAP1;
      UCSCTL6 = UCSCTL6 & (~XT1OFF);            //set XT1OFF=0 (bit0 = 0) --> enable XT1
                        
                        
      //use REFO for FLL 
      UCSCTL3 = (UCSCTL3 & ~(SELREF_7)) | (SELREF__REFOCLK);
      //use XT1CLK for ACLK
      UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__XT1CLK);      
      
      do
      {
        if (!(UCSCTL7 & XT1LFOFFG)) break;
        UCSCTL7 = UCSCTL7 & ~(0x0F);            // Clear XT2,XT1,DCO fault flags
        SFRIFG1 = SFRIFG1 & ~(0x02);            // Clear fault flags (SFRIFG1.OFIFG = 0)
        timeout--;
      }
      while ((SFRIFG1 & OFIFG) && timeout);      // Test oscillator fault flag

      UCSCTL6 = (UCSCTL6 & ~(1<<6)) & ~(1<<7);  //XT1Drive = 00; --> oscilator range 4-8 MHz      
      
      //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
      Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //Start the FLL, at the freq indicated by the config
                                                                        //constant USB_MCLK_FREQ
     
      XT2_Start(XT2DRIVE_0);                                          //Start the "USB crystal"
            
    } 
    else {
            #if defined (__MSP430F552x) || defined (__MSP430F550x)
                    P5SEL |= 0x10;                                    //enable XT1 pins
            #endif
    //Use the REFO oscillator to source the FLL and ACLK
    UCSCTL3 = SELREF__REFOCLK;
    UCSCTL4 = (UCSCTL4 & ~(SELA_7)) | (SELA__REFOCLK);

    //MCLK will be driven by the FLL (not by XT2), referenced to the REFO
    Init_FLL_Settle(USB_MCLK_FREQ / 1000, USB_MCLK_FREQ / 32768);   //set FLL (DCOCLK)

    XT1_Start(XT1DRIVE_0);                                          //Start the "USB crystal"
        
    }
}

/*  
 * ======== Init_Ports ========
 */
VOID Init_Ports (VOID)
{
    //Initialization of ports (all unused pins as outputs with low-level
    P1OUT = 0x00;
    P1DIR = 0x00;
    P2REN |= BIT0;                //Enable BUT2 pullup
    P2OUT = 0x01;
    P2DIR = 0x00;      
    P4OUT = 0x00;
    P4DIR = 0x00;
    P5OUT = 0x00;
    P5DIR = 0x00;
    P6OUT = 0x00;
    P6DIR = 0x00;
    PJDIR = BIT3;
    PJOUT |= BIT1;
    PJOUT &= ~BIT3;
}


/*  
 * ======== UNMI_ISR ========
 */
#pragma vector = UNMI_VECTOR
__interrupt VOID UNMI_ISR (VOID)
{
    switch (__even_in_range(SYSUNIV, SYSUNIV_BUSIFG ))
    {
        case SYSUNIV_NONE:
            __no_operation();
            break;
        case SYSUNIV_NMIIFG:
            __no_operation();
            break;
        case SYSUNIV_OFIFG:
            UCSCTL7 &= ~(DCOFFG + XT1LFOFFG + XT2OFFG); //Clear OSC flaut Flags fault flags
            SFRIFG1 &= ~OFIFG;                          //Clear OFIFG fault flag
            break;
        case SYSUNIV_ACCVIFG:
            __no_operation();
            break;
        case SYSUNIV_BUSIFG:
            SYSBERRIV = 0;                                      //clear bus error flag
            USB_disable();                                      //Disable
    }
}

/*  
 * ======== Init_Timer1 ========
 */
void Init_Timer1 (void)
{  
   TA1CTL |= TASSEL0 + MC0 + TACLR;              //ACLK for T1 source
                                                 //Up mode: Timer counts up to TAxCCR0
                                                 //reset timer A1                                         
   TA1CCTL0 |= CCIE;                             //Interrupt enabled
   TA1CCR0 = TIMER1_PERIOD;   
}

/*  
 * ======== TIMER1_ISR ========
 */
#pragma vector = TIMER1_A0_VECTOR
__interrupt VOID TIMER1_A0_ISR (VOID)
{
    if (!(SFRIFG1 & OFIFG)) 
    { 
      LED1_TOGGLE;    
    }  
    else LED1_ON;
}

/*  
 * ======== LOW POWER ENTERING ========
 */
 void Low_Power_Entering(void)
{

    // Disable VUSB LDO and SLDO
    USBKEYPID   =     0x9628;           // set USB KEYandPID to 0x9628 
                                      // access to USB config registers enabled  
    USBPWRCTL &= ~(SLDOEN+VUSBEN);      // Disable the VUSB LDO and the SLDO
    USBKEYPID   =    0x9600;            // access to USB config registers disabled 
  
    __disable_interrupt();
    XT2_Stop();
    __bis_SR_register(LPM3_bits);       // Enter LPM3 
    __no_operation();
    while(1);
        
}


